@@ -34,9 +34,10 @@ from vis.tests import test_indexer
 from vis.tests import test_note_rest_indexer
 from vis.tests import test_ngram
 from vis.tests import test_repeat
-from vis.tests import test_aggregator
 from vis.tests import test_interval_indexer
 from vis.tests import test_frequency_experimenter
+from vis.tests import test_aggregator
+from vis.tests import test_barchart
 from vis.tests import test_offset
 from vis.tests import test_lilypond
 from vis.tests import test_indexed_piece
@@ -48,11 +49,10 @@ from vis.tests import test_workflow_experiments
 
 
 THE_TESTS = (# Indexer and Subclasses
-             test_indexer.INDEXER_HARDCORE_SUITE,
+             test_indexer.INDEXER_INIT_SUITE,
              test_indexer.INDEXER_1_PART_SUITE,
              test_indexer.INDEXER_MULTI_EVENT_SUITE,
              test_indexer.UNIQUE_OFFSETS_SUITE,
-             test_indexer.VERT_ALIGNER_SUITE,
              test_note_rest_indexer.NOTE_REST_INDEXER_SUITE,
              test_interval_indexer.INTERVAL_INDEXER_SHORT_SUITE,
              test_interval_indexer.INTERVAL_INDEXER_LONG_SUITE,
@@ -67,17 +67,17 @@ THE_TESTS = (# Indexer and Subclasses
              test_lilypond.PART_NOTES_SUITE,
              test_lilypond.LILYPOND_SUITE,
              # Experimenter and Subclasses
-             test_frequency_experimenter.FREQUENCY_FUNC_SUITE,
-             test_frequency_experimenter.FREQUENCY_RUN_SUITE,
+             test_frequency_experimenter.FREQUENCY_SUITE,
              test_aggregator.COLUMN_AGGREGATOR_SUITE,
-             # IndexedPiece and AggregatedPieces\
+             test_barchart.R_BAR_CHART_SUITE,
+             # IndexedPiece and AggregatedPieces
              test_indexed_piece.INDEXED_PIECE_SUITE_A,
              test_indexed_piece.INDEXED_PIECE_SUITE_B,
              test_indexed_piece.INDEXED_PIECE_PARTS_TITLES,
              test_aggregated_pieces.AGGREGATED_PIECES_SUITE,
              # WorkflowManager
              test_workflow.WORKFLOW_TESTS,
-             test_workflow.GET_DATA_FRAME,
+             test_workflow.FILTER_DATA_FRAME,
              test_workflow.EXPORT,
              test_workflow.EXTRA_PAIRS,
              test_workflow.SETTINGS,
