@@ -28,113 +28,115 @@
 Experimenters related to producing LilyPond-format output from the VIS Framework. Also refer to the
 :mod:`vis.analyzers.indexers.lilypond` module.
 
-The :class:`LilyPondIndexer` uses the :mod:`outputlilypond` module to produce a LilyPond file
+The :class:`LilyPondExperimenter` uses the :mod:`outputlilypond` module to produce a LilyPond file
 corresponding to the score.
 """
 
-# Disable "string statement has no effect"
-# pylint: disable=W0105
+# pylint: disable=pointless-string-statement
 
 from math import fsum
+from numpy import isnan, NaN  # pylint: disable=no-name-in-module
 import pandas
 from music21 import stream, note, duration
 import outputlilypond
 from outputlilypond import settings as oly_settings
-from vis.analyzers import indexer
+from vis.analyzers import experimenter
 
 
 def annotate_the_note(obj):
     """
-    Used by :class:`AnnotateTheNoteIndexer` to make a :class:`~music21.note.Note` object with the
-    annotation passed in. Take note (hahaha): the ``lily_invisible`` property is set to ``True``!
+    Used by :class:`AnnotateTheNoteExperimenter` to make a :class:`~music21.note.Note` object with
+    the annotation passed in. Take note (hahaha): the ``lily_invisible`` property is set to ``True``!
 
-    :param obj: A single-element :class:`Series` with the string to put as the ``lily_markup``
-        property of a new :class:`Note`.
-    :type obj: :class:`pandas.Series` of ``unicode``
+    :param obj: A string to put as the ``lily_markup`` property of a new :class:`Note`.
+    :type obj: basestring
 
     :returns: An annotated note.
     :rtype: :class:`music21.note.Note`
     """
-    post = note.Note()
-    post.lily_invisible = True
-    post.lily_markup = obj[0]
-    return post
+    if isinstance(obj, float) and isnan(obj):
+        return NaN
+    else:
+        post = note.Note()
+        post.lily_invisible = True
+        post.lily_markup = obj
+        return post
 
 
-class LilyPondIndexer(indexer.Indexer):
+class LilyPondExperimenter(experimenter.Experimenter):
     """
     Use the :mod:`outputlilypond` module to produce the LilyPond file that should produce a score
     of the input.
+
+    .. note:: Perhaps contrary to expectation, you must provide a :class:`music21.stream.Score` to
+        the :class:`LilyPondExperimenter`, and any part with annotations belong in the settings.
     """
 
-    required_score_type = stream.Score
+    required_score_type = 'stream.Score'
     """
-    You must provide a :class:`music21.stream.Score` to this Indexer.
+    This attribute allows :class:`IndexedPiece` to automatically import and provide the
+    :class:`Score` for :class:`LilyPondExperimenter`. Otherwise you would have to do this manually.
     """
 
-    possible_settings = [u'run_lilypond', u'output_pathname', u'annotation part']
+    possible_settings = ['run_lilypond', 'output_pathname', 'annotation part']
     """
-    Possible settings for the :class:`LilyPondIndexer` include:
+    Possible settings for the :class:`LilyPondExperimenter` include:
 
-    :keyword u'run_lilypond': Whether to run LilyPond; if ``False`` or omitted, simply produce the \
-        input file LilyPond requires.
-    :type u'run_lilypond': boolean
-
-    :keyword u'output_pathname': Pathname for the resulting LilyPond output file. If \
-        ``u'run_lilypond'`` is ``True``, you must include this setting. If u'run_lilypond' is \
-        ``False`` and you do not provide ``u'output_pathname'`` then the output file is returned \
+    :keyword boolean 'run_lilypond': Whether to run LilyPond; if ``False`` or omitted, simply
+        produce the input file LilyPond requires.
+    :keyword basestring 'output_pathname': Pathname for the resulting LilyPond output file. If
+        ``'run_lilypond'`` is ``True``, you must include this setting. If ``'run_lilypond'`` is
+        ``False`` and you do not provide ``'output_pathname'`` then the output file is returned
         by :meth:`run` as a ``unicode``.
-    :type u'output_pathname': ``basestring``
-
-    :keyword u'annotation_part': A :class:`Part` or list of :class:`Part` objects with annotation
+    :keyword 'annotation_part': A :class:`Part` or list of :class:`Part` objects with annotation
         instructions for :mod:`outputlilypond`. This :class:`Part` will be appended as last in
         the :class:`Score`.
-    :type u'annotation_part': :class:`music21.stream.Part` or list of :class:`Part`
+    :type 'annotation_part': :class:`music21.stream.Part` or list of :class:`Part`
     """
 
-    default_settings = {u'run_lilypond': False, u'output_pathname': None, u'annotation_part': None}
-    """
-    Default settings.
-    """
+    default_settings = {'run_lilypond': False, 'output_pathname': None, 'annotation_part': None}
 
     # error message for when settings say to run LilyPond, but we have no pathname
-    error_no_pathname = u'LilyPondIndexer cannot run LilyPond without saving output to a file.'
+    _MISSING_PATHNAME = u'LilyPondExperimenter missing required "output_pathname" setting'
 
-    def __init__(self, score, settings=None):
+    def __init__(self, index, settings=None):
         """
-        :param score: The :class:`Score` object to output to LilyPond.
-        :type score: singleton list of :class:`music21.stream.Score`
-
+        :param index: The :class:`Score` object to output to LilyPond.
+        :type index: singleton list of :class:`music21.stream.Score`
         :param settings: Your settings. There are no required settings.
-        :type settings: ``dict`` or :const:`None`
+        :type settings: dict or NoneType
 
-        :raises: :exc:`RuntimeError` if ``score`` is the wrong type.
-        :raises: :exc:`RuntimeError` if ``u'run_lilypond'`` is ``True`` but ``u'output_pathname'`` \
+        :raises: :exc:`RuntimeError` if ``index`` is the wrong type.
+        :raises: :exc:`RuntimeError` if ``'run_lilypond'`` is ``True`` but ``'output_pathname'``
             is unspecified.
         """
         settings = {} if settings is None else settings
         self._settings = {}
+
         # dealing with output_pathname is a little complicated...
         if u'output_pathname' in settings:
             self._settings[u'output_pathname'] = settings[u'output_pathname']
             if u'run_lilypond' in settings:
                 self._settings[u'run_lilypond'] = settings[u'run_lilypond']
         else:
-            self._settings[u'output_pathname'] = LilyPondIndexer.default_settings[u'output_pathname']
+            self._settings[u'output_pathname'] = LilyPondExperimenter.default_settings[u'output_pathname']
             if u'run_lilypond' in settings and settings[u'run_lilypond'] is True:
-                raise RuntimeError(LilyPondIndexer.error_no_pathname)
+                raise RuntimeError(LilyPondExperimenter._MISSING_PATHNAME)
+
         # if they didn't specify whether to run LilyPond
         if u'run_lilypond' not in self._settings:
-            self._settings[u'run_lilypond'] = LilyPondIndexer.default_settings[u'run_lilypond']
+            self._settings[u'run_lilypond'] = LilyPondExperimenter.default_settings[u'run_lilypond']
+
         # deal with the annotation_part
         if u'annotation_part' in settings:
             self._settings[u'annotation_part'] = settings[u'annotation_part']
             if not isinstance(self._settings[u'annotation_part'], list):
                 self._settings[u'annotation_part'] = [self._settings[u'annotation_part']]
         else:
-            self._settings[u'annotation_part'] = LilyPondIndexer.default_settings[u'annotation_part']
-        super(LilyPondIndexer, self).__init__(score, None)
-        # We won't use an indexer function; run() is just going to pass the Score to outputlilypond
+            self._settings[u'annotation_part'] = LilyPondExperimenter.default_settings[u'annotation_part']
+
+        super(LilyPondExperimenter, self).__init__(index, None)
+
         self._indexer_func = None
 
     def run(self):
@@ -142,50 +144,62 @@ class LilyPondIndexer(indexer.Indexer):
         Make a string with the LilyPond representation of each score. Run LilyPond, if we're
         supposed to.
 
-        :returns: A list of strings, where each string is the LilyPond-format representation of the
-            score that was in that index.
-        :rtype: ``list`` of ``unicode``
+        :returns: A string holding the LilyPond-format representation of the score and its
+            annotation parts.
+        :rtype: basestring
         """
         lily_setts = oly_settings.LilyPondSettings()
+
         # append analysis part, if present
         if self._settings[u'annotation_part'] is not None:
             for part in self._settings[u'annotation_part']:
-                self._score[0].insert(0, part)
+                self._index[0].insert(0, part)
+
         # because outputlilypond uses multiprocessing by itself, we'll just call it in series
-        the_score = outputlilypond.process_score(self._score[0], lily_setts)
+        the_score = outputlilypond.process_score(self._index[0], lily_setts)
+
         # output the score, if given a pathname
         if self._settings[u'output_pathname'] is not None:
             with open(self._settings[u'output_pathname'], 'w') as handle:
                 handle.write(the_score)
+
         # call LilyPond on each file, if required
         if self._settings[u'run_lilypond'] is True:
             outputlilypond.run_lilypond(self._settings[u'output_pathname'], lily_setts)
+
         return the_score
 
 
-class AnnotateTheNoteIndexer(indexer.Indexer):
+class AnnotateTheNoteExperimenter(experimenter.Experimenter):
     """
     Make a new :class:`~music21.note.Note` object with the input set to the ``lily_markup``
     property, the ``lily_invisible`` property set to ``True``, and everything else as a default
     :class:`Note`.
     """
 
-    required_score_type = pandas.Series
-    possible_settings = []
-    default_settings = {}
+    possible_settings = ['column']
+    """
+    Use the ``'column'`` setting to determine which column of the :class:`DataFrame` will be used
+    as the annotations for the notes in the outputted list of :class:`Series`.
+    """
 
-    def __init__(self, score, settings=None):
+    _MISSING_SETTING = 'AnnotateTheNoteExperimenter is missing the "column" setting.'
+
+    def __init__(self, index, settings=None):
         """
-        :param score: The input from which to produce a new index.
-        :type score: ``list`` of :class:`pandas.Series`
+        :param index: The input from which to produce a new index---the output of the
+            :class:`vis.analyzers.indexers.lilypond.AnnotationIndexer`.
+        :type index: :class:`pandas.DataFrame`
 
-        :param settings: Nothing.
-        :type settings: ``dict`` or :const:`None`
+        :param settings: A dictionary with the required setting.
 
-        :raises: :exc:`RuntimeError` if ``score`` is the wrong type.
-        :raises: :exc:`RuntimeError` if ``score`` is not a list of the same types.
+        :raises: :exc:`RuntimeError` if the ``'column'`` setting is not in ``settings``.
         """
-        super(AnnotateTheNoteIndexer, self).__init__(score, None)
+        super(AnnotateTheNoteExperimenter, self).__init__(index, None)
+        if 'column' not in settings:
+            raise RuntimeError(AnnotateTheNoteExperimenter._MISSING_SETTING)
+        else:
+            self._settings['column'] = settings['column']
         self._indexer_func = annotate_the_note
 
     def run(self):
@@ -195,14 +209,15 @@ class AnnotateTheNoteIndexer(indexer.Indexer):
         :returns: A list of the new indices. The index of each :class:`Series` corresponds to the
             index of the :class:`Part` used to generate it, in the order specified to the
             constructor. Each element in the :class:`Series` is a ``basestring``.
-        :rtype: ``list`` of :class:`pandas.Series`
+        :rtype: list of :class:`pandas.Series`
         """
-        # Calculate each part separately:
-        combinations = [[x] for x in xrange(len(self._score))]
-        return self._do_multiprocessing(combinations)
+        self._index = self._index[self._settings['column']]
+        results = [self._index[s].map(self._indexer_func) for s in self._index]
+        results = [x.dropna() for x in results]
+        return results
 
 
-class PartNotesIndexer(indexer.Indexer):
+class PartNotesExperimenter(experimenter.Experimenter):
     """
     From a :class:`Series` full of :class:`Note` objects, craft a :class:`music21.stream.Part`. The
     offset of each :class:`Note` in the output matches its index in the input :class:`Series`, and
@@ -214,7 +229,7 @@ class PartNotesIndexer(indexer.Indexer):
 
     required_score_type = pandas.Series
     default_settings = {}
-    possible_settings = [u'part_names']
+    possible_settings = ['part_names']
     """
     :param part_names: Names for the annotation parts, in order. If there are more part names than
         parts, extra names will be ignored. If there are fewer part names than parts, some parts
@@ -222,21 +237,20 @@ class PartNotesIndexer(indexer.Indexer):
     :type part_names: list of basestring
     """
 
-    _IMPOSSIBLE_QUARTERLENGTH = u'Impossible \'quarterLength\': %s.'
+    _IMPOSSIBLE_QUARTERLENGTH = 'Impossible \'quarterLength\': {}.'
 
     def __init__(self, score, settings=None):
         """
         :param score: The input from which to produce a new index.
-        :type score: ``list`` of :class:`pandas.Series` of :class:`music21.note.Note`
+        :type score: list of :class:`pandas.Series` of :class:`music21.note.Note`
 
         :param settings: Nothing.
-        :type settings: ``dict`` or :const:`None`
+        :type settings: dict or NoneType
 
         :raises: :exc:`RuntimeError` if ``score`` is the wrong type.
         :raises: :exc:`RuntimeError` if ``score`` is not a list of the same types.
         """
-        super(PartNotesIndexer, self).__init__(score, None)
-        self._indexer_func = None
+        super(PartNotesExperimenter, self).__init__(score, None)
         if settings is not None and 'part_names' in settings:
             self._settings['part_names'] = settings['part_names']
         else:
@@ -253,7 +267,7 @@ class PartNotesIndexer(indexer.Indexer):
         :type end_o: ``float``
 
         :returns: The ``quarterLength`` values that fill the whole duration (see below).
-        :rtype: ``list`` of ``float``
+        :rtype: list of float
 
         The algorithm tries to use as few ``quarterLength`` values as possible, but prefers multiple
         values to a single dotted value. The longest single value is ``4.0`` (a whole note).
@@ -287,7 +301,7 @@ class PartNotesIndexer(indexer.Indexer):
                 else:
                     return [highest] + the_solver(ql_remains - highest)
             else:
-                raise RuntimeError(PartNotesIndexer._IMPOSSIBLE_QUARTERLENGTH % (end_o - start_o))
+                raise RuntimeError(PartNotesExperimenter._IMPOSSIBLE_QUARTERLENGTH.format(end_o - start_o))
 
         return the_solver(float(end_o) - float(start_o))
 
@@ -321,7 +335,7 @@ class PartNotesIndexer(indexer.Indexer):
         for i in xrange(in_len):
             qls = None
             try:
-                qls = PartNotesIndexer._fill_space_between_offsets(in_part[i].offset,
+                qls = PartNotesExperimenter._fill_space_between_offsets(in_part[i].offset,
                                                                    in_part[i + 1].offset)
             except stream.StreamException:  # when we try to access the note after the last
                 qls = [1.0]
@@ -357,7 +371,7 @@ class PartNotesIndexer(indexer.Indexer):
         :type in_part: :class:`pandas.Series`
         """
         if 0.0 != in_part.index[0]:
-            durations = PartNotesIndexer._fill_space_between_offsets(0.0, in_part.index[0])
+            durations = PartNotesExperimenter._fill_space_between_offsets(0.0, in_part.index[0])
             offsets = [max(0.0, sum(durations[:i])) for i in xrange(len(durations))]
             for i, offset in enumerate(offsets):
                 in_part[offset] = note.Rest(quarterLength=durations[i])
@@ -372,11 +386,11 @@ class PartNotesIndexer(indexer.Indexer):
         :returns: A list of the new indices. The index of each :class:`Part` corresponds to the
             index of the :class:`Series` used to generate it, in the order specified to the
             constructor. Each element in the :class:`Part` is a :class:`Note`.
-        :rtype: ``list`` of :class:`music21.stream.Part`
+        :rtype: list of :class:`music21.stream.Part`
         """
         post = []
-        for i, each_series in enumerate(self._score):
-            each_series = PartNotesIndexer._prepend_rests(each_series)
+        for i, each_series in enumerate(self._index):
+            each_series = PartNotesExperimenter._prepend_rests(each_series)
             new_part = stream.Part()
             new_part.lily_analysis_voice = True
             if 'part_names' in self._settings:
@@ -391,5 +405,5 @@ class PartNotesIndexer(indexer.Indexer):
             for off, obj in each_series.iteritems():
                 new_part.insert(off, obj)
 
-            post.append(PartNotesIndexer._set_durations(new_part))
+            post.append(PartNotesExperimenter._set_durations(new_part))
         return post
