@@ -23,13 +23,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #--------------------------------------------------------------------------------------------------
 """
-.. codeauthor:: Christopher Antila <crantila@fedoraproject.org>
+.. codeauthor:: Christopher Antila <christopher@antila.ca>
 
 Indexer to find k-part any-object n-grams.
 """
 
-# Turn off "string statement has no effect" warning; the strings are for Sphinx!
-# pylint: disable=W0105
+# pylint: disable=pointless-string-statement
 
 import pandas
 from vis.analyzers import indexer
@@ -48,112 +47,107 @@ class NGramIndexer(indexer.Indexer):
     There is no relationship between the number of index types, though there must be at least one
     "vertical" index.
 
-    The settings given to :meth:`__init__` specify which index values in ``score`` are
-    horizontal or vertical intervals. They will be added in the n-gram in the order specified, so
-    if the ``u'vertical'`` setting is ``[4, 1, 3]`` for lists of intervals, then for each\
-    vertical event, objects will be listed in that order.
+    The ``'horizontal'`` and ``'vertical'`` settings determine which columns of the ``score``
+    :class:`DataFrame` are included in the n-gram output. They are added to the n-gram in the order
+    specified, so if the ``'vertical'`` setting is
+    ``[('noterest.NoteRestIndexer', '1'), ('noterest.NoteRestIndexer', '0')]``, this will put the
+    lower part (with index ``'1'``) before the higher part (with index ``'0'``). Note that both the
+    indexer's name and the part-combination name must be included.
 
-    In the output, groups of "vertical" events are enclosed in brackets, while groups of
-    "horizontal" events are enclosed in parentheses.
+    This is an example minimum ``settings`` dictionary for making interval 3-grams:::
 
-    For cases where there is only one index in a particular direction, you can avoid printing the
-    brackets or parentheses by setting the ``u'mark singles'`` setting to False (though the
-    default is ``True``).
+        {'vertical': [('interval.IntervalIndexer', '0,1')],
+         'horizontal': [('interval.HorizontalIntervalIndexer', '1')],
+         'n': 3}
+
+    In the output, groups of "vertical" events are normally enclosed in brackets, while groups of
+    "horizontal" events are enclosed in parentheses. For cases where there is only one index in a
+    particular direction, you can avoid printing the brackets or parentheses by setting the
+    ``'mark singles'`` setting to ``False`` (the default is ``True``).
 
     If you want n-grams to terminate when finding one or several particular values, you can specify
-    this with the ``u'terminator'`` setting.
+    this with the ``'terminator'`` setting.
 
-    To show that a horizontal event continues, we use ``u'_'`` by default, but you can set this
-    separately, for example to ``u'P1'`` ``u'0'``, as seems appropriate. Note that the default
-    :class:`WorkflowManager` is set to override this setting by dynamically adjusting to whether
-    interval quality is set to True or False or if the user chooses to pass a custom string for
-    this setting.
+    To show that a horizontal event continues, we use ``'_'`` by default, but you can set this
+    separately, for example to ``'P1'`` ``'0'``, as seems appropriate. Note that the default
+    :class:`WorkflowManager` overrides this setting by dynamically adjusting for interval quality,
+    and also offers a ``'continuer'`` setting of its own, which is passed to this indexer.
 
     You can also use the :class:`NGramIndexer` to collect "stacks" of single vertical events. If
     you provide indices of intervals above a lowest part, for example, these "stacks" become the
-    figured bass signature of a single moment. Set :obj:`u'n'` to 1 for this feature. Horizontal
-    events are obviously ignored.
+    figured bass signature of a single moment. Set ``'n'`` to ``1`` for this feature. Horizontal
+    events are obviously ignored in this case.
     """
 
-    required_score_type = pandas.Series
-    "The :class:`NGramIndexer` requires :class:`pandas.Series` as input."
+    required_score_type = 'pandas.DataFrame'
 
-    possible_settings = [u'horizontal', u'vertical', u'n', u'mark_singles', u'terminator',
-                         u'continuer']
+    possible_settings = ['horizontal', 'vertical', 'n', 'mark_singles', 'terminator', 'continuer']
     """
     A list of possible settings for the :class:`NGramIndexer`.
 
-    :keyword u'horizontal': The parts to consider as "horizontal."
-    :type u'horizontal': ``list`` of ``int``
-    :keyword u'vertical': The parts to consider as "vertical."
-    :type u'vertical': ``list`` of ``int``
-    :keyword u'n': The number of "vertical" events per n-gram.
-    :type u'n': ``int``
-    :keyword u'mark_singles': Whether to use delimiters around a direction's events when
+    :keyword 'horizontal': Selectors for the parts to consider as "horizontal."
+    :type 'horizontal': list of (basestring, basestring) tuples
+    :keyword 'vertical': Selectors for the parts to consider as "vertical."
+    :type 'vertical': list of (basestring, basestring) tuples
+    :keyword 'n': The number of "vertical" events per n-gram.
+    :type 'n': int
+    :keyword 'mark_singles': Whether to use delimiters around a direction's events when
         there is only one event in that direction (e.g., the "horizontal" maps only the activity
-        of a single voice). (You may also use ``u'mark singles'``).
-    :type u'mark_singles': ``bool``
-    :keyword u'terminator': Do not find an n-gram with a vertical item that contains any of these
+        of a single voice). (You may also use ``'mark singles'``).
+    :type 'mark_singles': bool
+    :keyword 'terminator': Do not find an n-gram with a vertical item that contains any of these
         values.
-    :type u'terminator': ``list`` of ``basestring``
-    :keyword u'continuer': When there is no "horizontal" event that corresponds to a vertical
+    :type 'terminator': list of basestring
+    :keyword 'continuer': When there is no "horizontal" event that corresponds to a vertical
         event, this is printed instead, to show that the previous "horizontal" event continues.
-    :type u'continuer': ``basestring``
+    :type 'continuer': basestring
     """
 
-    default_settings = {u'mark_singles': True, u'horizontal': [], u'terminator': [],
-                        u'continuer': u'_'}
-    "A :obj:`dict` of default settings for the :class:`NGramIndexer`."
+    default_settings = {'mark_singles': True, 'horizontal': [], 'terminator': [], 'continuer': '_'}
+
+    _MISSING_SETTINGS = 'NGramIndexer requires "vertical" and "n" settings'
+    _N_VALUE_TOO_LOW = 'NGramIndexer requires an "n" value of at least 1'
 
     def __init__(self, score, settings=None):
         """
-        :param score: A list of the "horizontal" and "vertical" indices to use for n-grams. You \
-            can put the "horizontal" and "vertical" indices anywhere in the list, so long as you \
-            use settings to specify the order.
-        :type score: ``list`` of :class:`pandas.Series`
-        :param settings: Required and optional settings. See descriptions in \
+        :param score: The :class:`DataFrame` to use for preparing n-grams. You must ensure the
+            :class:`DataFrame` has the columns indicated in the ``settings``, or the :meth:`run`
+            method will fail.
+        :type score: :class:`pandas.DataFrame`
+        :param dict settings: Required and optional settings. See descriptions in
             :const:`possible_settings`.
-        :type settings: ``dict``
 
         :raises: :exc:`RuntimeError` if ``score`` is the wrong type.
         :raises: :exc:`RuntimeError` if ``score`` is not a list of the same types.
         :raises: :exc:`RuntimeError` if required settings are not present in ``settings``.
-        :raises: :exc:`RuntimeError` if ``u'n'`` is less than ``1``.
+        :raises: :exc:`RuntimeError` if ``'n'`` is less than ``1``.
         """
         # Check all required settings are present in the "settings" argument.
-        if settings is None or u'vertical' not in settings or u'n' not in settings:
-            msg = u'NGramIndexer requires "vertical" and "n" settings'
-            raise RuntimeError(msg)
-        elif settings[u'n'] < 1:
-            msg = u'NGramIndexer requires an "n" value of at least 1'
-            raise RuntimeError(msg)
+        if settings is None or 'vertical' not in settings or 'n' not in settings:
+            raise RuntimeError(NGramIndexer._MISSING_SETTINGS)
+        elif settings['n'] < 1:
+            raise RuntimeError(NGramIndexer._N_VALUE_TOO_LOW)
         else:
             self._settings = {}
-            self._settings[u'vertical'] = settings[u'vertical']
-            self._settings[u'n'] = settings[u'n']
-            self._settings[u'horizontal'] = settings[u'horizontal'] if u'horizontal' in settings \
-                else NGramIndexer.default_settings[u'horizontal']
-            if u'mark singles' in settings:
-                self._settings[u'mark_singles'] = settings[u'mark singles']
-            elif u'mark_singles' in settings:
-                self._settings[u'mark_singles'] = settings[u'mark_singles']
+            self._settings['vertical'] = settings['vertical']
+            self._settings['n'] = settings['n']
+            self._settings['horizontal'] = (settings['horizontal'] if 'horizontal' in settings
+                else NGramIndexer.default_settings['horizontal'])
+            if 'mark singles' in settings:
+                self._settings['mark_singles'] = settings['mark singles']
+            elif 'mark_singles' in settings:
+                self._settings['mark_singles'] = settings['mark_singles']
             else:
-                self._settings[u'mark_singles'] = NGramIndexer.default_settings[u'mark_singles']
-            self._settings[u'terminator'] = settings[u'terminator'] if u'terminator' in settings \
-                else NGramIndexer.default_settings[u'terminator']
-            self._settings[u'continuer'] = settings[u'continuer'] if u'continuer' in settings \
-                else NGramIndexer.default_settings[u'continuer']
+                self._settings['mark_singles'] = NGramIndexer.default_settings['mark_singles']
+            self._settings['terminator'] = (settings['terminator'] if 'terminator' in settings
+                else NGramIndexer.default_settings['terminator'])
+            self._settings['continuer'] = (settings['continuer'] if 'continuer' in settings
+                else NGramIndexer.default_settings['continuer'])
 
-        # Change "TemplateIndexer" to the current class name. The superclass will handle the
-        # "score" and "mpc" arguments, but you should have processed "settings" above, so it should
-        # not be sent to the superclass constructor.
         super(NGramIndexer, self).__init__(score, None)
 
-        # not using it
-        self._indexer_func = None
-
     @staticmethod
-    def _format_thing(things, m_singles, markers=(u'[', u']'), terminator=None):
+    def _format_thing(things, m_singles, markers=('[', ']'), terminator=None):
         """
         Format unicode objects by concatenating them with a space between and the appropriate
         grouping symbol, if relevant. This method is used by _format_vert() and _format_horiz().
@@ -183,19 +177,19 @@ class NGramIndexer(indexer.Indexer):
             post.append(markers[0])
             for obj in things:
                 if obj in terminator:
-                    raise RuntimeWarning(u'hit a terminator')
+                    raise RuntimeWarning('hit a terminator')
                 else:
                     post.append(unicode(obj))
-                    post.append(u' ')
-            post = post[:-1] # remove last space
+                    post.append(' ')
+            post = post[:-1]  # remove last space
             post.append(markers[1])
         elif things[0] in terminator:
-            raise RuntimeWarning(u'hit a terminator')
+            raise RuntimeWarning('hit a terminator')
         elif m_singles:
             post.extend([markers[0], unicode(things[0]), markers[1]])
         else:
             post.append(things[0])
-        return u''.join(post)
+        return ''.join(post)
 
     @staticmethod
     def _format_vert(verts, m_singles, terminator=None):
@@ -219,7 +213,7 @@ class NGramIndexer(indexer.Indexer):
 
         :raises: RuntimeWarning, if the one of the events is a "terminator."
         """
-        return NGramIndexer._format_thing(verts, m_singles, (u'[', u']'), terminator)
+        return NGramIndexer._format_thing(verts, m_singles, ('[', ']'), terminator)
 
     @staticmethod
     def _format_horiz(horizs, m_singles, terminator=None):
@@ -243,72 +237,100 @@ class NGramIndexer(indexer.Indexer):
 
         :raises: RuntimeWarning, if the one of the events is a "terminator."
         """
-        return NGramIndexer._format_thing(horizs, m_singles, (u'(', u')'), terminator)
+        return NGramIndexer._format_thing(horizs, m_singles, ('(', ')'), terminator)
+
+    def _make_column_label(self):
+        """
+        Make the part-combination column label for the returned DataFrame's MultiIndex. This
+        involves a rather complex coordination between the "vertical," "horizontal," and
+        "mark_singles" settings.
+
+        Refer to the automated tests for examples of what happens.
+        """
+        verts = ['{}'.format(x[1]) for x in self._settings['vertical']]
+        if len(verts) > 1 or self._settings['mark_singles']:
+            verts = '[{}]'.format(' '.join(verts))
+        else:
+            verts = ' '.join(verts)
+
+        if 'horizontal' in self._settings and len(self._settings['horizontal']) > 0:
+            horizs = ['{}'.format(x[1]) for x in self._settings['horizontal']]
+            if len(horizs) > 1 or self._settings['mark_singles']:
+                horizs = '({})'.format(' '.join(horizs))
+            else:
+                horizs = ' '.join(horizs)
+            return ['{} {}'.format(verts, horizs)]
+        else:
+            return [verts]
 
     def run(self):
         """
         Make an index of k-part n-grams of anything.
 
-        :returns: A single-item list with the new index.
-        :rtype: ``list`` of :class:`pandas.Series`
+        :returns: A single-column :class:`~pandas.DataFrame` with the new index.
         """
-        # TODO: pylint says there are too many branches; it's right
+        # NOTE: in an incredible stroke of luck, the VIS 1 run() algorithm works without change
+        #       for the VIS 2.0 release...
+        # - So in a future 2.x-series point release, we can add "true" multidimensional functionality
+        #   while retaining the existing 'horizontal' and 'vertical' method of naming the dimensions.
+        #   In other words, we'll break the API at release 2.0 while retaining the algorithm, and
+        #   add new features along with a new algorithm later, without breaking the API.
 
         post = []
         post_offsets = []
 
         # for the formatting methods
-        m_singles = self._settings[u'mark_singles']
-        term = self._settings[u'terminator']
+        m_singles = self._settings['mark_singles']
+        term = self._settings['terminator']
 
         # Order the parts as specified. We have to track "i" and "name" separately so we have a new
         # order for the dict but can keep self._score straight. We'll use these tuples to keep
         # vertical and horizontal events separated in the DataFrame with a MultiIndex
         events = {}
-        for i, name in enumerate(self._settings[u'vertical']):
-            events[(u'v', i)] = self._score[name]
-        for i, name in enumerate(self._settings[u'horizontal']):
-            events[(u'h', i)] = self._score[name]
+        for i, name in enumerate(self._settings['vertical']):
+            events[('v', i)] = self._score[name].dropna()
+        for i, name in enumerate(self._settings['horizontal']):
+            events[('h', i)] = self._score[name].dropna()
 
         # Make the MultiIndex and DataFrame with all events
         events = pandas.DataFrame(events, columns=pandas.MultiIndex.from_tuples(events.keys()))
 
         # Fill in all "vertical" NaN values with the previous value
-        for i in events[u'v'].columns:
+        for i in events['v'].columns:
             # NB: still have to test the fix, as stated in issue 261
-            events.update(events.loc[:,(u'v', i)].fillna(method=u'ffill'))
+            events.update(events.loc[:, ('v', i)].fillna(method='ffill'))
 
         # Fill in all "horizontal" NaN values with the continuer
-        if u'h' in events:
-            for i in events[u'h'].columns:
+        if 'h' in events:
+            for i in events['h'].columns:
                 # NB: still have to test the fix, as stated in issue 261
-                events.update(events.loc[:,(u'h', i)].fillna(value=self._settings[u'continuer']))
+                events.update(events.loc[:, ('h', i)].fillna(value=self._settings['continuer']))
 
         # Iterate the offsets
         for i in xrange(len(events)):
             loop_post = None
             try:
                 # first vertical event
-                loop_post = [NGramIndexer._format_vert(list(events[u'v'].iloc[i].sort_index()),
-                                                            m_singles,
-                                                            term)]
+                loop_post = [NGramIndexer._format_vert(list(events['v'].iloc[i].sort_index()),
+                                                       m_singles,
+                                                       term)]
             except RuntimeWarning:  # we hit a terminator
                 continue
             try:
-                for j in xrange(self._settings[u'n'] - 1):  # iterate to the end of 'n'
+                for j in xrange(self._settings['n'] - 1):  # iterate to the end of 'n'
                     k = i + j + 1  # the index we need
                     ilp = None  # it means "Inner Loop Post"
-                    if u'h' in events:  # are there "horizontal" events?
-                        ilp = [u' ',
-                               NGramIndexer._format_horiz(list(events[u'h'].iloc[k].sort_index()),
+                    if 'h' in events:  # are there "horizontal" events?
+                        ilp = [' ',
+                               NGramIndexer._format_horiz(list(events['h'].iloc[k].sort_index()),
                                                           m_singles),
-                               u' ',
-                               NGramIndexer._format_vert(list(events[u'v'].iloc[k].sort_index()),
+                               ' ',
+                               NGramIndexer._format_vert(list(events['v'].iloc[k].sort_index()),
                                                          m_singles,
                                                          term)]
                     else:
-                        ilp = [u' ',
-                               NGramIndexer._format_vert(list(events[u'v'].iloc[k].sort_index()),
+                        ilp = [' ',
+                               NGramIndexer._format_vert(list(events['v'].iloc[k].sort_index()),
                                                          m_singles,
                                                          term)]
                     loop_post.extend(ilp)
@@ -317,7 +339,9 @@ class NGramIndexer(indexer.Indexer):
                     break
                 else:  # we hit a terminator
                     continue
-            post.append(u''.join(loop_post))
+            post.append(''.join(loop_post))
             post_offsets.append(events.index[i])
 
-        return [pandas.Series(post, post_offsets)]
+        # prepare the part-combination labels
+        combos = self._make_column_label()
+        return self.make_return(combos, [pandas.Series(post, post_offsets)])
