@@ -7,7 +7,7 @@
 # Filename:               controllers/indexers/repeat.py
 # Purpose:                Indexers that somehow consider repetition.
 #
-# Copyright (C) 2013 Christopher Antila
+# Copyright (C) 2013, 2014 Christopher Antila
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -23,12 +23,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #--------------------------------------------------------------------------------------------------
 """
-.. codeauthor:: Christopher Antila <crantila@fedoraproject.org>
+.. codeauthor:: Christopher Antila <christopher@antila.ca>
 
 Indexers that consider repetition in any way.
 """
 
-import numpy
+from numpy import nan
 import pandas
 from vis.analyzers import indexer
 
@@ -37,26 +37,27 @@ class FilterByRepeatIndexer(indexer.Indexer):
     """
     If the same event occurs many times in a row, remove all occurrences but the one with the \
     lowest ``offset`` value (i.e., the "first" event).
+
+    Because of how a :class:`DataFrame`'s index works, many of the events that would have been
+    filtered will instead be replaced with :const:`numpy.NaN`. Please be careful that the behaviour
+    of this indexer matches your expectations.
     """
 
-    required_score_type = pandas.Series
-    "The :class:`FilterByRepeatIndexer` requires :class:`pandas.Series` as input."
+    required_score_type = 'pandas.Series'
 
     def __init__(self, score, settings=None):
         """
-        :param score: The indices from which to remove consecutive identical events.
-        :type score: :obj:`list` of :obj:`pandas.Series`
+        :param score: The indices from which to remove consecutive identical events. There must be
+            at least one part in the score.
+        :type score: :class:`pandas.DataFrame` or list of :class:`pandas.Series`
         :param settings: This indexer uses no settings, so this is ignored.
-        :type settings: :obj:`dict` or :obj:`None`
+        :type settings: dict or NoneType
 
-        :raises: :exc:`RuntimeError` if :obj:`score` is the wrong type.
-        :raises: :exc:`RuntimeError` if :obj:`score` is not a list of the same types.
+        :raises: :exc:`RuntimeError` if ``score`` is the wrong type.
+        :raises: :exc:`RuntimeError` if ``score`` is not a list of the same types.
         """
         super(FilterByRepeatIndexer, self).__init__(score, None)
 
-        # If self._score is a Stream (subclass), change to a list of types you want to process
-        self._types = []
-
         # This Indexer uses pandas magic, not an _indexer_func().
         self._indexer_func = None
 
@@ -64,10 +65,8 @@ class FilterByRepeatIndexer(indexer.Indexer):
         """
         Make a new index of the piece, removing any event that is identical to the preceding.
 
-        :returns: A list of the new indices. The index of each Series corresponds to the index of
-            the Part used to generate it, in the order specified to the constructor. Each element
-            in the Series is a :obj:`basestring`.
-        :rtype: :obj:`list` of :obj:`pandas.Series`
+        :returns: A :class:`DataFrame` of the new indices.
+        :rtype: :class:`pandas.DataFrame`
         """
         # I'm relying on pandas' efficiency. In the future, maybe we should use multiprocessing?
         post = []
@@ -84,6 +83,9 @@ class FilterByRepeatIndexer(indexer.Indexer):
                     axe_me.append(offset)
                 prev_off = offset
             for axed in axe_me:
-                part[axed] = numpy.nan
+                part[axed] = nan
             post.append(part.dropna())
-        return post
+
+        # prepare the proper return type
+        combinations = [[x] for x in xrange(len(self._score))]
+        return self.make_return([unicode(x)[1:-1] for x in combinations], post)
