@@ -7,7 +7,7 @@
 # Filename:               models_tests/test_indexed_piece.py
 # Purpose:                Tests for models/indexed_piece.py.
 #
-# Copyright (C) 2013 Christopher Antila
+# Copyright (C) 2013, 2014 Christopher Antila
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -26,6 +26,7 @@
 Tests for :py:class:`~vis.models.indexed_piece.IndexedPiece`.
 """
 
+import os
 from unittest import TestCase, TestLoader
 from mock import call, patch, MagicMock, Mock
 import pandas
@@ -35,6 +36,9 @@ from vis.analyzers.indexers import noterest
 from vis.analyzers.experimenter import Experimenter
 from vis.models.indexed_piece import IndexedPiece, _find_piece_title, _find_part_names, OpusWarning
 
+# find pathname to the 'vis' directory
+import vis
+VIS_PATH = vis.__path__[0]
 
 # pylint: disable=R0904
 # pylint: disable=C0111
@@ -67,13 +71,37 @@ class TestIndexedPieceA(TestCase):
         non_analyzer = Mock()
         self.assertRaises(TypeError, self.ind_piece.get_data, non_analyzer)
 
-    def test_get_data_1(self):
-        # get data for an Indexer requiring a Score
+    @patch('vis.models.indexed_piece.IndexedPiece._import_score')
+    def test_get_data_1a(self, mock_is):
+        """get data for an Indexer requiring a [Part]"""
         mock_indexer_cls = type('MockIndexer', (Indexer,), {})
-        mock_indexer_cls.required_score_type = music21.stream.Part
+        mock_indexer_cls.required_score_type = 'stream.Part'
         mock_indexer_cls.run = MagicMock()
-        mock_indexer_cls.run.return_value = u'ahhh!'
-        self.assertEquals(u'ahhh!', self.ind_piece.get_data([mock_indexer_cls]))
+        mock_indexer_cls.run.return_value = 'ahhh!'
+        expected = 'ahhh!'
+        mock_is.return_value = MagicMock(spec_set=music21.stream.Score)
+        mock_is.return_value.parts = [MagicMock(spec_set=music21.stream.Part)]
+
+        actual = self.ind_piece.get_data([mock_indexer_cls])
+
+        self.assertEquals(expected, actual)
+        mock_is.assert_called_once_with(known_opus=False)
+        mock_indexer_cls.run.assert_called_once_with()
+
+    @patch('vis.models.indexed_piece.IndexedPiece._import_score')
+    def test_get_data_1a(self, mock_is):
+        """get data for an Indexer requiring a [Part]"""
+        mock_indexer_cls = type('MockIndexer', (Indexer,), {})
+        mock_indexer_cls.required_score_type = 'stream.Score'
+        mock_indexer_cls.run = MagicMock()
+        mock_indexer_cls.run.return_value = 'ahhh!'
+        expected = 'ahhh!'
+        mock_is.return_value = MagicMock(spec_set=music21.stream.Score)
+
+        actual = self.ind_piece.get_data([mock_indexer_cls])
+
+        self.assertEquals(expected, actual)
+        mock_is.assert_called_once_with(known_opus=False)
         mock_indexer_cls.run.assert_called_once_with()
 
     def test_get_data_2(self):
@@ -97,7 +125,7 @@ class TestIndexedPieceA(TestCase):
         first_indexer_cls.__init__ = first_init
         first_indexer_cls.run = MagicMock()
         first_indexer_cls.run.return_value = [14]
-        first_indexer_cls.required_score_type = music21.stream.Part
+        first_indexer_cls.required_score_type = 'stream.Part'
         second_indexer_cls = type('MockIndexer', (Indexer,), {})
         second_init = MagicMock()
         second_init.return_value = None
@@ -119,7 +147,7 @@ class TestIndexedPieceA(TestCase):
         first_indexer_cls.__init__ = first_init
         first_indexer_cls.run = MagicMock()
         first_indexer_cls.run.return_value = [14]
-        first_indexer_cls.required_score_type = music21.stream.Part
+        first_indexer_cls.required_score_type = 'stream.Part'
         second_indexer_cls = type('MockIndexer', (Indexer,), {})
         second_init = MagicMock()
         second_init.return_value = None
@@ -144,7 +172,7 @@ class TestIndexedPieceA(TestCase):
         mock_indexer_cls.__init__ = mock_init
         mock_indexer_cls.run = MagicMock()
         mock_indexer_cls.run.return_value = [14]
-        mock_indexer_cls.required_score_type = music21.stream.Part
+        mock_indexer_cls.required_score_type = 'stream.Part'
         mock_experimenter_cls = type('MockIndexer', (Experimenter,), {})
         exp_init = MagicMock()
         exp_init.return_value = None
@@ -224,7 +252,7 @@ class TestIndexedPieceA(TestCase):
         with patch.object(IndexedPiece, u'_import_score') as mock_is:
             with patch.object(IndexedPiece, u'_type_verifier') as mock_tv:
                 mock_ind = MagicMock()
-                mock_ind.required_score_type = music21.stream.Part
+                mock_ind.required_score_type = 'stream.Part'
                 self.ind_piece.get_data([mock_ind], known_opus='horse')
                 mock_is.assert_called_once_with(known_opus='horse')
 
@@ -382,7 +410,7 @@ class TestPartsAndTitles(TestCase):
    # _find_part_names() and _find_piece_title() methods.
    # NOTE: not testing "Sanctus.krn" because it's an Opus, and we can't deal with them yet.
     def test_bwv77(self):
-        path = u'vis/tests/corpus/bwv77.mxl'
+        path = os.path.join(VIS_PATH, 'tests', 'corpus', 'bwv77.mxl')
         expected_title = u'bwv77'
         expected_parts = [u'Soprano', u'Alto', u'Tenor', u'Bass']
         the_score = music21.converter.parse(path)
@@ -392,7 +420,7 @@ class TestPartsAndTitles(TestCase):
         self.assertSequenceEqual(expected_parts, actual_parts)
 
     def test_jos2308_krn(self):
-        path = u'vis/tests/corpus/Jos2308.krn'
+        path = os.path.join(VIS_PATH, 'tests', 'corpus', 'Jos2308.krn')
         expected_title = u'Jos2308'
         expected_parts = [u'spine_3', u'spine_2', u'spine_1', u'spine_0']
         the_score = music21.converter.parse(path)
@@ -402,7 +430,7 @@ class TestPartsAndTitles(TestCase):
         self.assertSequenceEqual(expected_parts, actual_parts)
 
     def test_kyrie(self):
-        path = u'vis/tests/corpus/Kyrie.krn'
+        path = os.path.join(VIS_PATH, 'tests', 'corpus', 'Kyrie.krn')
         expected_title = u'Kyrie'
         expected_parts = [u'spine_4', u'spine_3', u'spine_2', u'spine_1', u'spine_0']
         the_score = music21.converter.parse(path)
@@ -412,7 +440,7 @@ class TestPartsAndTitles(TestCase):
         self.assertSequenceEqual(expected_parts, actual_parts)
 
     def test_madrigal51(self):
-        path = u'vis/tests/corpus/madrigal51.mxl'
+        path = os.path.join(VIS_PATH, 'tests', 'corpus', 'madrigal51.mxl')
         expected_title = u'madrigal51'
         expected_parts = [u'Canto', u'Alto', u'Tenor', u'Quinto', u'Basso', u'Continuo']
         the_score = music21.converter.parse(path)
@@ -422,7 +450,7 @@ class TestPartsAndTitles(TestCase):
         self.assertSequenceEqual(expected_parts, actual_parts)
 
     def test_sinfony(self):
-        path = u'vis/tests/corpus/sinfony.md'
+        path = os.path.join(VIS_PATH, 'tests', 'corpus', 'sinfony.md')
         expected_title = u'Messiah'
         expected_parts = [u'Violino I', u'Violino II', u'Viola', u'Bassi']
         the_score = music21.converter.parse(path)
@@ -432,7 +460,7 @@ class TestPartsAndTitles(TestCase):
         self.assertSequenceEqual(expected_parts, actual_parts)
 
     def test_opus76(self):
-        path = u'vis/tests/corpus/sqOp76-4-i.midi'
+        path = os.path.join(VIS_PATH, 'tests', 'corpus', 'sqOp76-4-i.midi')
         expected_title = u'sqOp76-4-i'
         expected_parts = [u'Part 1', u'Part 2', u'Part 3', u'Part 4']
         the_score = music21.converter.parse(path)
@@ -442,7 +470,7 @@ class TestPartsAndTitles(TestCase):
         self.assertSequenceEqual(expected_parts, actual_parts)
 
     def test_bwv2(self):
-        path = u'vis/tests/corpus/bwv2.xml'
+        path = os.path.join(VIS_PATH, 'tests', 'corpus', 'bwv2.xml')
         expected_title = u'bwv2'
         expected_parts = [u'Soprano', u'Alto', u'Tenor', u'Bass']
         the_score = music21.converter.parse(path)
