@@ -32,6 +32,7 @@
 import unittest
 import mock
 import copy
+from numpy import NaN
 import pandas
 from music21 import base, stream, duration, note, converter, clef
 from vis.analyzers import indexer
@@ -42,35 +43,177 @@ def fake_indexer_func(ecks):
     return unicode(ecks)
 
 
-class TestIndexerHardcore(unittest.TestCase):
+class TestIndexerInit(unittest.TestCase):
     # accessing TestIndexer._indexer_func is part of the test
     # pylint: disable=W0212
-    def test_indexer_hardcore_1(self):
+    def test_indexer_init_1(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 1: stream.Part, given a list of Part
+        class TestIndexer(indexer.Indexer):
+            # Minimum required changes!
+            required_score_type = 'stream.Part'
+        test_score = [stream.Part(), stream.Part()]
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, list))
+        self.assertSequenceEqual(test_score, test_ind._score)
+
+    def test_indexer_init_2a(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 2a: stream.Part, given a Score (two parts in the Score)
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'stream.Part'
+        test_score = stream.Score([stream.Part(), stream.Part()])
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, list))
+        self.assertSequenceEqual(test_score, test_ind._score)
+
+    def test_indexer_init_2b(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 2b: stream.Part, given a Score (no parts in the Score)
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'stream.Part'
+        test_score = stream.Score([])
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, list))
+        self.assertSequenceEqual(test_score, test_ind._score)
+
+    def test_indexer_init_3(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 3: stream.Score, given a Score
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'stream.Score'
+        test_score = stream.Score([stream.Part(), stream.Part()])
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, stream.Score))
+        self.assertSequenceEqual(test_score, test_ind._score)
+
+    def test_indexer_init_4(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 4: pandas.Series, given a list of Series
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        test_score = [pandas.Series(), pandas.Series()]
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, list))
+        self.assertSequenceEqual(test_score, test_ind._score)
+
+    def test_indexer_init_5a(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 5a: pandas.Series, given a DataFrame
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        the_index = pandas.MultiIndex.from_tuples([('NRI', '0'), ('NRI', '1')],
+                                                  names=['Indexer', 'Parts'])
+        series_list = [pandas.Series([2]), pandas.Series([1])]
+        test_score = pandas.DataFrame(series_list, index=the_index).T
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, list))
+        self.assertSequenceEqual(list(series_list[0].index), list(test_ind._score[0].index))
+        self.assertSequenceEqual(list(series_list[1].index), list(test_ind._score[1].index))
+        self.assertSequenceEqual(list(series_list[0].values), list(test_ind._score[0].values))
+        self.assertSequenceEqual(list(series_list[1].values), list(test_ind._score[1].values))
+
+    def test_indexer_init_5b(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 5b: pandas.Series, given a DataFrame (but there are two indexers of results!)
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        the_tuples = [('NRI', '0'), ('NRI', '1'), ('HII', '0'), ('HII', '1')]
+        the_index = pandas.MultiIndex.from_tuples(the_tuples, names=['Indexer', 'Parts'])
+        series_list = [pandas.Series([i]) for i in [2, 4, 6, 8]]  # who do we appreciate?
+        test_score = pandas.DataFrame(series_list, index=the_index).T
+        self.assertRaises(IndexError, TestIndexer, test_score)
+        try:
+            TestIndexer(test_score)
+        except IndexError as inderr:
+            self.assertEqual(indexer.Indexer._INIT_INDEX_ERR, inderr.message)
+
+    def test_indexer_init_5c(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 5c: pandas.Series, given a DataFrame (but there is no MultiIndex)
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        series_dict = {str(i): pandas.Series([i]) for i in [2, 4, 6, 8]}
+        test_score = pandas.DataFrame(series_dict)
+        self.assertRaises(IndexError, TestIndexer, test_score)
+        try:
+            TestIndexer(test_score)
+        except IndexError as inderr:
+            self.assertEqual(indexer.Indexer._INIT_INDEX_ERR, inderr.message)
+
+    def test_indexer_init_5d(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 5d: pandas.Series, given a DataFrame (but there are no indexers)
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        test_score = pandas.DataFrame()
+        self.assertRaises(IndexError, TestIndexer, test_score)
+        try:
+            TestIndexer(test_score)
+        except IndexError as inderr:
+            self.assertEqual(indexer.Indexer._INIT_INDEX_ERR, inderr.message)
+
+    def test_indexer_init_5e(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 5e: pandas.Series, given a DataFrame---make sure NaN values are dropped from the Series
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        the_index = pandas.MultiIndex.from_tuples([('NRI', '0'), ('NRI', '1')],
+                                                  names=['Indexer', 'Parts'])
+        series_input = [pandas.Series([1, NaN, 2, NaN, 3, NaN, 4, NaN, NaN, 5]),
+                        pandas.Series([42, 43, 44, NaN, NaN, NaN])]
+        # NB: the first Series will have a strange index because of the "internal" NaNs
+        expected = [pandas.Series([1, 2, 3, 4, 5], index=[0, 2, 4, 6, 9]),
+                    pandas.Series([42, 43, 44])]
+        test_score = pandas.DataFrame(series_input, index=the_index).T
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, list))
+        self.assertSequenceEqual(list(expected[0].index), list(test_ind._score[0].index))
+        self.assertSequenceEqual(list(expected[1].index), list(test_ind._score[1].index))
+        self.assertSequenceEqual(list(expected[0].values), list(test_ind._score[0].values))
+        self.assertSequenceEqual(list(expected[1].values), list(test_ind._score[1].values))
+
+    def test_indexer_init_6(self):
+        # The first six tests ensure __init__() accepts the six valid types of "score" argument.
+        # 6: pandas.DataFrame, given a DataFrame
+        class TestIndexer(indexer.Indexer):
+            required_score_type = 'pandas.DataFrame'
+        the_index = pandas.MultiIndex.from_tuples([('NRI', '0'), ('NRI', '1')],
+                                                  names=['Indexer', 'Parts'])
+        series_list = [pandas.Series([2]), pandas.Series([1])]
+        test_score = pandas.DataFrame(series_list, index=the_index).T
+        test_ind = TestIndexer(test_score)
+        self.assertTrue(isinstance(test_ind._score, pandas.DataFrame))
+        self.assertSequenceEqual(list(series_list[0].index), list(test_ind._score['NRI']['0'].index))
+        self.assertSequenceEqual(list(series_list[1].index), list(test_ind._score['NRI']['1'].index))
+        self.assertSequenceEqual(list(series_list[0].values), list(test_ind._score['NRI']['0'].values))
+        self.assertSequenceEqual(list(series_list[1].values), list(test_ind._score['NRI']['1'].values))
+
+    def test_indexer_init_7(self):
         # That calling Indexer.__init__() with the wrong type results in the proper error message.
         class TestIndexer(indexer.Indexer):
             # Class with bare minimum changes, since we can't instantiate Indexer directly
-            required_score_type = stream.Stream
+            required_score_type = 'stream.Part'
 
         test_parts = [pandas.Series()]
         settings = {}
         self.assertRaises(TypeError, TestIndexer, test_parts, settings)
-        error_msg = unicode("<class 'vis.tests.test_indexer.TestIndexer'> requires "
-                            "<class 'music21.stream.Stream'> objects, not <class 'pandas.core"
-                            ".series.Series'>")
+        error_msg = indexer.Indexer._INIT_TYPE_ERR.format(u"<class 'vis.tests.test_indexer.TestIndexer'>",
+                                                          u"stream.Part")
         try:
             TestIndexer(test_parts, settings)
         except TypeError as err:
             self.assertEqual(err.args[0], error_msg)
 
-    def test_indexer_hardcore_2(self):
+    def test_indexer_init_8(self):
         # That _do_multiprocessing() passes the correct arguments to stream_indexer()
         class TestIndexer(indexer.Indexer):
             # Class with bare minimum changes, since we can't instantiate Indexer directly
-            required_score_type = stream.Stream
+            required_score_type = 'stream.Part'
             def run(self):
                 self._do_multiprocessing([[0]])
         # the actual testing
-        test_parts = [stream.Stream()]
+        test_parts = [stream.Part()]
         settings = {}
         # prepare mocks
         with mock.patch(u'vis.analyzers.indexer.stream_indexer') as mpi_mock:
@@ -81,7 +224,24 @@ class TestIndexerHardcore(unittest.TestCase):
             # check results
             mpi_mock.assert_called_once_with(0, test_parts, fake_indexer_func, None)
 
-    def test_indexer_hardcore_3(self):
+    def test_indexer_init_9(self):
+        # That calling Indexer.__init__() with required_score_type set to an invalid value results
+        # in the proper error message.
+        class TestIndexer(indexer.Indexer):
+            # Class with bare minimum changes, since we can't instantiate Indexer directly
+            required_score_type = stream.Part
+
+        test_parts = [pandas.Series()]
+        settings = {}
+        self.assertRaises(TypeError, TestIndexer, test_parts, settings)
+        error_msg = unicode('<class \'vis.tests.test_indexer.TestIndexer\'> has an incorrectly-set '
+                            '"required_score_type"')
+        try:
+            TestIndexer(test_parts, settings)
+        except TypeError as err:
+            self.assertEqual(err.args[0], error_msg)
+
+    def test_indexer_init_10(self):
         # when no "types" is specified, make sure it returns everything in the Stream
         # setup the input stream
         the_stream = stream.Stream()
@@ -503,60 +663,60 @@ class TestMpiUniqueOffsets(unittest.TestCase):
         self.assertSequenceEqual(expected, actual)
 
 
-class TestMpiVertAligner(unittest.TestCase):
-    def test_mpi_vert_aligner_1(self):
-        in_list = [[1]]
-        expected = [[1]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
+class TestMakeReturn(unittest.TestCase):
+    def test_make_return_1(self):
+        # 1: the usual case
+        # prepare
+        violini = pandas.Series([x for x in xrange(10)])
+        violinii = pandas.Series([x + 10 for x in xrange(10)])
+        viola = pandas.Series([x + 20 for x in xrange(10)])
+        cello = pandas.Series([x + 30 for x in xrange(10)])
+        names = [u'Violin I', u'Violin II', u'Viola', u'Violoncello']
+        parts = [violini, violinii, viola, cello]
+        class QuartetIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        # run
+        test_ind = QuartetIndexer(parts)
+        actual = test_ind.make_return(names, parts)
+        # check
+        self.assertEqual(u'test_indexer.QuartetIndexer', actual.columns.levels[0][0])
 
-    def test_mpi_vert_aligner_2(self):
-        in_list = [[1, 2]]
-        expected = [[1], [2]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
+    def test_make_return_2(self):
+        # 2: more parts than names
+        # prepare
+        clarinet = pandas.Series([x for x in xrange(10)])
+        tuba = pandas.Series([x + 10 for x in xrange(10)])
+        names = [u'Clarinet']
+        parts = [clarinet, tuba]
+        class SadIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        # run
+        test_ind = SadIndexer(parts)
+        self.assertRaises(IndexError, test_ind.make_return, names, parts)
+        # check
+        try:
+            test_ind.make_return(names, parts)
+        except IndexError as inderr:
+            self.assertEqual(indexer.Indexer._MAKE_RETURN_INDEX_ERR, inderr.message)
 
-    def test_mpi_vert_aligner_3(self):
-        in_list = [[1], [2]]
-        expected = [[1, 2]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
+    def test_make_return_3(self):
+        # 3: more names than parts
+        # prepare
+        clarinet = pandas.Series([x for x in xrange(10)])
+        #tuba = pandas.Series([x + 10 for x in xrange(10)])
+        names = [u'Clarinet', u'Tuba']
+        parts = [clarinet]
+        class SadIndexer(indexer.Indexer):
+            required_score_type = 'pandas.Series'
+        # run
+        test_ind = SadIndexer(parts)
+        self.assertRaises(IndexError, test_ind.make_return, names, parts)
+        # check
+        try:
+            test_ind.make_return(names, parts)
+        except IndexError as inderr:
+            self.assertEqual(indexer.Indexer._MAKE_RETURN_INDEX_ERR, inderr.message)
 
-    def test_mpi_vert_aligner_4(self):
-        in_list = [[1, 2], [1, 2]]
-        expected = [[1, 1], [2, 2]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
-
-    def test_mpi_vert_aligner_5(self):
-        in_list = [[1, 2], [5]]
-        expected = [[1, 5], [2]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
-
-    def test_mpi_vert_aligner_6(self):
-        in_list = [[1, 2, 3], [1, 2, 3], [1, 2, 3]]
-        expected = [[1, 1, 1], [2, 2, 2], [3, 3, 3]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
-
-    def test_mpi_vert_aligner_7(self):
-        in_list = [[1, 2, 3], [1], [1, 2, 3]]
-        expected = [[1, 1, 1], [2, 2], [3, 3]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
-
-    def test_mpi_vert_aligner_8(self):
-        in_list = [[1, 2, 3], [1], [1, 2]]
-        expected = [[1, 1, 1], [2, 2], [3]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
-
-    def test_mpi_vert_aligner_9(self):
-        in_list = [[1], [1], [1]]
-        expected = [[1, 1, 1]]
-        actual = indexer.mpi_vert_aligner(in_list)
-        self.assertSequenceEqual(expected, actual)
 
 #--------------------------------------------------------------------------------------------------#
 # Definitions                                                                                      #
@@ -564,5 +724,5 @@ class TestMpiVertAligner(unittest.TestCase):
 INDEXER_1_PART_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestIndexerSinglePart)
 INDEXER_MULTI_EVENT_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestIndexerMultiEvent)
 UNIQUE_OFFSETS_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestMpiUniqueOffsets)
-VERT_ALIGNER_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestMpiVertAligner)
-INDEXER_HARDCORE_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestIndexerHardcore)
+INDEXER_INIT_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestIndexerInit)
+MAKE_RETURN_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestMakeReturn)
