@@ -22,18 +22,21 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #--------------------------------------------------------------------------------------------------
+"""
+Tests for the 'indexers.lilypond' and 'experimenters.lilypond' modules.
+"""
 
-# allow "no docstring" for everything
-# pylint: disable=C0111
 # allow "too many public methods" for TestCase
-# pylint: disable=R0904
-
+# pylint: disable=too-many-public-methods
 
 import unittest
 import mock
+from numpy import isnan, NaN  # pylint: disable=no-name-in-module
 import pandas
 from music21 import note, stream
 from vis.analyzers.indexers import lilypond
+from vis.analyzers.experimenters.lilypond import PartNotesExperimenter, AnnotateTheNoteExperimenter, \
+    LilyPondExperimenter, annotate_the_note
 
 
 class TestAnnotationIndexer(unittest.TestCase):
@@ -54,33 +57,63 @@ class TestAnnotationIndexer(unittest.TestCase):
                   pandas.Series(['how', 'we']),
                   pandas.Series(['do', 'it']),
                   pandas.Series(['baby', 'yeah'])]
-        expected = [pandas.Series(['_\\markup{ "this" }', '_\\markup{ "is" }']),
-                    pandas.Series(['_\\markup{ "how" }', '_\\markup{ "we" }']),
-                    pandas.Series(['_\\markup{ "do" }', '_\\markup{ "it" }']),
-                    pandas.Series(['_\\markup{ "baby" }', '_\\markup{ "yeah" }'])]
+        expected = {'0': pandas.Series(['_\\markup{ "this" }', '_\\markup{ "is" }']),
+                    '1': pandas.Series(['_\\markup{ "how" }', '_\\markup{ "we" }']),
+                    '2': pandas.Series(['_\\markup{ "do" }', '_\\markup{ "it" }']),
+                    '3': pandas.Series(['_\\markup{ "baby" }', '_\\markup{ "yeah" }'])}
         actual = lilypond.AnnotationIndexer(in_val).run()
-        for i in xrange(len(expected)):
+        self.assertIsInstance(actual, pandas.DataFrame)
+        actual = actual['lilypond.AnnotationIndexer']
+        for i in expected:
             self.assertSequenceEqual(list(expected[i].index), list(actual[i].index))
             self.assertSequenceEqual(list(expected[i]), list(actual[i]))
 
 
-class TestAnnotateTheNoteIndexer(unittest.TestCase):
+class TestAnnotateTheNoteExperimenter(unittest.TestCase):
+    """Tests for AnnotateTheNoteExperimenter."""
+
     def test_ind_func_1(self):
-        in_val = pandas.Series([u'_\\markup{ "my shirt" }'])
-        actual = lilypond.annotate_the_note(in_val)
+        """That the indexing function, annotate_the_note(), works properly."""
+        in_val = u'_\\markup{ "my shirt" }'
+        actual = annotate_the_note(in_val)
         self.assertTrue(isinstance(actual, note.Note))
         self.assertTrue(hasattr(actual, u'lily_invisible'))
         self.assertTrue(hasattr(actual, u'lily_markup'))
         self.assertEqual(True, actual.lily_invisible)
         self.assertEqual(u'_\\markup{ "my shirt" }', actual.lily_markup)
 
-    def test_class_1(self):
+    def test_ind_func_2(self):
+        """That the indexing function, annotate_the_note(), works properly with NaN input."""
+        in_val = NaN
+        actual = annotate_the_note(in_val)
+        self.assertTrue(isnan(actual))
+
+    def test_init_1(self):
+        """That __init__() works properly."""
+        actual = AnnotateTheNoteExperimenter('some DF', {'column': 'Muhly'})
+        self.assertEqual('Muhly', actual._settings['column'])
+
+    def test_init_2(self):
+        """That __init__() raises a RuntimeError when the 'column' setting isn't given."""
+        self.assertRaises(RuntimeError, AnnotateTheNoteExperimenter, 'some DF', {'row': 'the zoo'})
+        try:
+            AnnotateTheNoteExperimenter('some DF', {'row': 'the zoo'})
+        except RuntimeError as run_err:
+            self.assertEqual(AnnotateTheNoteExperimenter._MISSING_SETTING, run_err.message)
+
+    def test_run_1(self):
+        """That run() works properly when all the annotations are at the same index/offset."""
         annotes = [['_\\markup{ "this" }', '_\\markup{ "is" }'],
                    ['_\\markup{ "how" }', '_\\markup{ "we" }'],
                    ['_\\markup{ "do" }', '_\\markup{ "it" }'],
                    ['_\\markup{ "baby" }', '_\\markup{ "yeah" }']]
-        in_val = [pandas.Series(l) for l in annotes]
-        actual = lilypond.AnnotateTheNoteIndexer(in_val).run()
+        in_val = pandas.DataFrame([pandas.Series(x) for x in annotes],
+                                  index=[['this', 'this', 'this', 'this'],
+                                         ['1', '2', '3', '4']]).T
+
+        actual = AnnotateTheNoteExperimenter(in_val, {'column': 'this'}).run()
+
+        self.assertEqual(len(annotes), len(actual))
         for i, each_part in enumerate(actual):
             for j, each_note in enumerate(each_part):
                 self.assertTrue(isinstance(each_note, note.Note))
@@ -89,17 +122,44 @@ class TestAnnotateTheNoteIndexer(unittest.TestCase):
                 self.assertEqual(True, each_note.lily_invisible)
                 self.assertEqual(annotes[i][j], each_note.lily_markup)
 
+    def test_run_2(self):
+        """That run() works properly when the annotations have a different index/offset."""
+        annotes = [['_\\markup{ "this" }', '_\\markup{ "is" }'],
+                   ['_\\markup{ "how" }', '_\\markup{ "we" }'],
+                   ['_\\markup{ "do" }', '_\\markup{ "it" }'],
+                   ['_\\markup{ "baby" }', '_\\markup{ "yeah" }']]
+        in_series = [pandas.Series(annotes[0], index=[0.0, 1.0]),
+                     pandas.Series(annotes[1], index=[0.0, 0.5]),
+                     pandas.Series(annotes[2], index=[0.5, 1.0]),
+                     pandas.Series(annotes[3], index=[0.25, 1.25])]
+        in_val = pandas.DataFrame(in_series,
+                                  index=[['this', 'this', 'this', 'this'],
+                                         ['1', '2', '3', '4']]).T
 
-class TestPartNotesIndexer(unittest.TestCase):
+        actual = AnnotateTheNoteExperimenter(in_val, {'column': 'this'}).run()
+
+        self.assertEqual(len(annotes), len(actual))
+        for i, each_part in enumerate(actual):
+            for j, each_note in enumerate(each_part):
+                self.assertTrue(isinstance(each_note, note.Note))
+                self.assertTrue(hasattr(each_note, u'lily_invisible'))
+                self.assertTrue(hasattr(each_note, u'lily_markup'))
+                self.assertEqual(True, each_note.lily_invisible)
+                self.assertEqual(annotes[i][j], each_note.lily_markup)
+                 # check the offsets are still right
+                self.assertEqual(in_series[i].index[j], each_part.index[j])
+
+
+class TestPartNotesExperimenter(unittest.TestCase):
     # NB: this is to help mock this method later
-    FSBO_PATH = u'vis.analyzers.indexers.lilypond.PartNotesIndexer._fill_space_between_offsets'
+    FSBO_PATH = 'vis.analyzers.experimenters.lilypond.PartNotesExperimenter._fill_space_between_offsets'
     # the fill_space_between_offsets() tests were taken from vis7
     def test_fsbo_1(self):
         # pylint: disable=W0212
         in_1 = 0.0
         in_2 = 1.0
         expected = [1.0]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -108,7 +168,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 0.0
         in_2 = 4.0
         expected = [4.0]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -117,7 +177,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 0.0
         in_2 = 5.0
         expected = [4.0, 1.0]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -126,7 +186,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 0.0
         in_2 = 8.0
         expected = [4.0, 4.0]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -135,7 +195,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 0.0
         in_2 = 9.0
         expected = [4.0, 4.0, 1.0]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -144,7 +204,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 4.5
         in_2 = 5.0
         expected = [0.5]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -153,7 +213,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 7693.5
         in_2 = 7703.0
         expected = [4.0, 4.0, 1.0, 0.5]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -162,7 +222,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 0.0
         in_2 = 3.96875
         expected = [2.0, 1.0, 0.5, 0.25, 0.125, 0.0625, 0.03125]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -171,7 +231,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_1 = 3.96875
         in_2 = 7.9375
         expected = [2.0, 1.0, 0.5, 0.25, 0.125, 0.0625, 0.03125]
-        actual = lilypond.PartNotesIndexer._fill_space_between_offsets(in_1, in_2)
+        actual = PartNotesExperimenter._fill_space_between_offsets(in_1, in_2)
         self.assertEqual(len(expected), len(actual))
         self.assertSequenceEqual(expected, actual)
 
@@ -184,9 +244,9 @@ class TestPartNotesIndexer(unittest.TestCase):
             in_val[i].offset = in_offsets[i]
         in_val.lily_analysis_voice = 42
         expected = [(0.0, 4.0), (4.0, 1.0)]
-        with mock.patch(TestPartNotesIndexer.FSBO_PATH) as mock_fsbo:
+        with mock.patch(TestPartNotesExperimenter.FSBO_PATH) as mock_fsbo:
             mock_fsbo.side_effect = [[4.0]]
-            actual = lilypond.PartNotesIndexer._set_durations(in_val)  # pylint: disable=W0212
+            actual = PartNotesExperimenter._set_durations(in_val)  # pylint: disable=W0212
             self.assertEqual(len(in_offsets) - 1, mock_fsbo.call_count)
         self.assertEqual(len(expected), len(actual))
         for i, obj in enumerate(actual):
@@ -204,9 +264,9 @@ class TestPartNotesIndexer(unittest.TestCase):
             in_val[i].offset = in_offsets[i]
         in_val.lily_instruction = 66
         expected = [(0.0, 2.0), (2.0, 1.0), (3.0, 1.0)]
-        with mock.patch(TestPartNotesIndexer.FSBO_PATH) as mock_fsbo:
+        with mock.patch(TestPartNotesExperimenter.FSBO_PATH) as mock_fsbo:
             mock_fsbo.side_effect = [[2.0, 1.0]]
-            actual = lilypond.PartNotesIndexer._set_durations(in_val)  # pylint: disable=W0212
+            actual = PartNotesExperimenter._set_durations(in_val)  # pylint: disable=W0212
             self.assertEqual(len(in_offsets) - 1, mock_fsbo.call_count)
         self.assertEqual(len(expected), len(actual))
         for i, obj in enumerate(actual):
@@ -225,9 +285,9 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_val.lily_analysis_voice = 42
         in_val.lily_instruction = 66
         expected = [(0.0, 2.0), (2.0, 0.5), (2.5, 0.25), (2.75, 1.0)]
-        with mock.patch(TestPartNotesIndexer.FSBO_PATH) as mock_fsbo:
+        with mock.patch(TestPartNotesExperimenter.FSBO_PATH) as mock_fsbo:
             mock_fsbo.side_effect = [[2.0, 0.5, 0.25]]
-            actual = lilypond.PartNotesIndexer._set_durations(in_val)  # pylint: disable=W0212
+            actual = PartNotesExperimenter._set_durations(in_val)  # pylint: disable=W0212
             self.assertEqual(len(in_offsets) - 1, mock_fsbo.call_count)
         self.assertEqual(len(expected), len(actual))
         for i, obj in enumerate(actual):
@@ -245,9 +305,9 @@ class TestPartNotesIndexer(unittest.TestCase):
             in_val[i].offset = in_offsets[i]
         expected = [(3.96875, 2.0), (5.96875, 1.0), (6.96875, 0.5), (7.46875, 0.25),
                     (7.71875, 0.125), (7.84375, 0.0625), (7.90625, 0.03125), (7.9375, 1.0)]
-        with mock.patch(TestPartNotesIndexer.FSBO_PATH) as mock_fsbo:
+        with mock.patch(TestPartNotesExperimenter.FSBO_PATH) as mock_fsbo:
             mock_fsbo.side_effect = [[2.0, 1.0, 0.5, 0.25, 0.125, 0.0625, 0.03125]]
-            actual = lilypond.PartNotesIndexer._set_durations(in_val)  # pylint: disable=W0212
+            actual = PartNotesExperimenter._set_durations(in_val)  # pylint: disable=W0212
             self.assertEqual(len(in_offsets) - 1, mock_fsbo.call_count)
         self.assertEqual(len(expected), len(actual))
         for i, obj in enumerate(actual):
@@ -267,7 +327,7 @@ class TestPartNotesIndexer(unittest.TestCase):
             in_val.append(obj)
         in_val = [pandas.Series(in_val, index=[0.0, 2.75])]
         expected = [(0.0, 2.0), (2.0, 0.5), (2.5, 0.25), (2.75, 1.0)]
-        actual = lilypond.PartNotesIndexer(in_val).run()[0]
+        actual = PartNotesExperimenter(in_val).run()[0]
         # Verify...
         # ... that the result is a Part with the proper LilyPond attributes
         self.assertTrue(isinstance(actual, stream.Part))
@@ -299,7 +359,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_val = [pandas.Series(in_val, index=[0.0, 2.75])]
         setts = {'part_names': ['the name']}
         expected = [(0.0, 2.0), (2.0, 0.5), (2.5, 0.25), (2.75, 1.0)]
-        actual = lilypond.PartNotesIndexer(in_val, setts).run()[0]
+        actual = PartNotesExperimenter(in_val, setts).run()[0]
         # Verify...
         # ... that the result is a Part with the proper LilyPond attributes
         self.assertTrue(isinstance(actual, stream.Part))
@@ -324,7 +384,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         # simple: add one thing in front of one other thing
         in_val = pandas.Series(['first'], index=[2.0])
         expected = pandas.Series([note.Rest(quarterLength=2.0), 'first'], index=[0.0, 2.0])
-        actual = lilypond.PartNotesIndexer._prepend_rests(in_val)  # pylint: disable=protected-access
+        actual = PartNotesExperimenter._prepend_rests(in_val)  # pylint: disable=protected-access
         self.assertSequenceEqual(list(expected.index), list(actual.index))
         self.assertSequenceEqual(list(expected.values), list(actual.values))
 
@@ -332,7 +392,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         # simple: everything's already right
         in_val = pandas.Series(['first'], index=[0.0])
         expected = pandas.Series(['first'], index=[0.0])
-        actual = lilypond.PartNotesIndexer._prepend_rests(in_val)  # pylint: disable=protected-access
+        actual = PartNotesExperimenter._prepend_rests(in_val)  # pylint: disable=protected-access
         self.assertSequenceEqual(list(expected.index), list(actual.index))
         self.assertSequenceEqual(list(expected.values), list(actual.values))
 
@@ -342,7 +402,7 @@ class TestPartNotesIndexer(unittest.TestCase):
                                index=[0.0, 0.25, 400.369, 1024.1024])
         expected = pandas.Series(['first', 'second', 'third', 'fourth'],
                                  index=[0.0, 0.25, 400.369, 1024.1024])
-        actual = lilypond.PartNotesIndexer._prepend_rests(in_val)  # pylint: disable=protected-access
+        actual = PartNotesExperimenter._prepend_rests(in_val)  # pylint: disable=protected-access
         self.assertSequenceEqual(list(expected.index), list(actual.index))
         self.assertSequenceEqual(list(expected.values), list(actual.values))
 
@@ -351,7 +411,7 @@ class TestPartNotesIndexer(unittest.TestCase):
         in_val = pandas.Series(['first'], index=[6.0])
         expected = pandas.Series([note.Rest(quarterLength=4.0), note.Rest(quarterLength=2.0), 'first'],
                                  index=[0.0, 4.0, 6.0])
-        actual = lilypond.PartNotesIndexer._prepend_rests(in_val)  # pylint: disable=protected-access
+        actual = PartNotesExperimenter._prepend_rests(in_val)  # pylint: disable=protected-access
         self.assertSequenceEqual(list(expected.index), list(actual.index))
         self.assertSequenceEqual(list(expected.values), list(actual.values))
 
@@ -363,7 +423,7 @@ class TestPartNotesIndexer(unittest.TestCase):
                                   note.Rest(quarterLength=1.0),
                                   'first'],
                                  index=[0.0, 4.0, 6.0, 7.0])
-        actual = lilypond.PartNotesIndexer._prepend_rests(in_val)  # pylint: disable=protected-access
+        actual = PartNotesExperimenter._prepend_rests(in_val)  # pylint: disable=protected-access
         self.assertSequenceEqual(list(expected.index), list(actual.index))
         self.assertSequenceEqual(list(expected.values), list(actual.values))
 
@@ -379,44 +439,46 @@ class TestPartNotesIndexer(unittest.TestCase):
                                   'third',
                                   'fourth'],
                                  index=[0.0, 4.0, 6.0, 7.0, 7.25, 400.369, 1024.1024])
-        actual = lilypond.PartNotesIndexer._prepend_rests(in_val)  # pylint: disable=protected-access
+        actual = PartNotesExperimenter._prepend_rests(in_val)  # pylint: disable=protected-access
         self.assertSequenceEqual(list(expected.index), list(actual.index))
         self.assertSequenceEqual(list(expected.values), list(actual.values))
 
 
-class TestLilyPondIndexer(unittest.TestCase):
+class TestLilyPondExperimenter(unittest.TestCase):
+    """Tests for the LilyPondExperimenter."""
+
     @mock.patch('vis.analyzers.indexer.Indexer.__init__', new=lambda x, y, z: None)
     def test_init_1(self):
-        # output_pathname unspecified; run lily (RuntimeError)
+        """output_pathname unspecified; run lily (RuntimeError)"""
         setts = {u'run_lilypond': True}
-        self.assertRaises(RuntimeError, lilypond.LilyPondIndexer, 12, setts)
+        self.assertRaises(RuntimeError, LilyPondExperimenter, 12, setts)
 
     @mock.patch('vis.analyzers.indexer.Indexer.__init__', new=lambda x, y, z: None)
     def test_init_2(self):
-        # output_pathname unspecified; don't run lily; have annotation part
+        """output_pathname unspecified; don't run lily; have annotation part"""
         setts = {u'run_lilypond': False, u'annotation_part': 42}
         expected = {u'run_lilypond': False, u'annotation_part': [42], u'output_pathname': None}
-        actual = lilypond.LilyPondIndexer(12, setts)
+        actual = LilyPondExperimenter(12, setts)
         self.assertEqual(expected, actual._settings)  # pylint: disable=W0212
 
     @mock.patch('vis.analyzers.indexer.Indexer.__init__', new=lambda x, y, z: None)
     def test_init_3(self):
-        # output_pathname specified; run lily; no annotation part
+        """output_pathname specified; run lily; no annotation part"""
         setts = {u'run_lilypond': True, u'output_pathname': u'PATH!'}
         expected = {u'run_lilypond': True, u'annotation_part': None, u'output_pathname': u'PATH!'}
-        actual = lilypond.LilyPondIndexer(12, setts)
+        actual = LilyPondExperimenter(12, setts)
         self.assertEqual(expected, actual._settings)  # pylint: disable=W0212
 
     @mock.patch('vis.analyzers.indexer.Indexer.__init__', new=lambda x, y, z: None)
     def test_init_4(self):
-        # output_pathname specified; don't run lily; two annotation parts
+        """output_pathname specified; don't run lily; two annotation parts"""
         setts = {u'run_lilypond': False, u'output_pathname': u'PATH!', u'annotation_part': [42, 52]}
         exp_setts = {u'run_lilypond': False, u'annotation_part': [42, 52], u'output_pathname': u'PATH!'}
-        actual = lilypond.LilyPondIndexer(12, setts)
+        actual = LilyPondExperimenter(12, setts)
         self.assertEqual(exp_setts, actual._settings)  # pylint: disable=W0212
 
     def test_run_1(self):
-        # with annotation_part; without output_pathname; not run_lilypond
+        """with annotation_part; without output_pathname; not run_lilypond"""
         # prepare mocks
         mock_open = mock.mock_open()
         mock_score_cls = type('MockIndexer', (stream.Score,), {})
@@ -428,13 +490,13 @@ class TestLilyPondIndexer(unittest.TestCase):
         expected = mock.MagicMock(spec_set=unicode)
         run_ly = mock.MagicMock()
         # run test
-        with mock.patch('vis.analyzers.indexers.lilypond.outputlilypond') as mock_oly:
+        with mock.patch('vis.analyzers.experimenters.lilypond.outputlilypond') as mock_oly:
             mock_oly.process_score.return_value = expected
             mock_oly.run_lilypond = run_ly
-            with mock.patch('vis.analyzers.indexers.lilypond.oly_settings') as mock_oly_s:
+            with mock.patch('vis.analyzers.experimenters.lilypond.oly_settings') as mock_oly_s:
                 mock_oly_s.LilyPondSettings.return_value = oly_setts
                 with mock.patch('__builtin__.open', mock_open):
-                    actual = lilypond.LilyPondIndexer([mock_score], setts).run()
+                    actual = LilyPondExperimenter([mock_score], setts).run()
         # verify results
             mock_oly.process_score.assert_called_once_with(mock_score, oly_setts)
         self.assertEqual(0, mock_open.call_count)
@@ -443,7 +505,7 @@ class TestLilyPondIndexer(unittest.TestCase):
         self.assertEqual(expected, actual)
 
     def test_run_2(self):
-        # without annotation_part; with output_pathname; do run_lilypond
+        """without annotation_part; with output_pathname; do run_lilypond"""
         # prepare mocks
         mock_open = mock.mock_open()
         mock_score_cls = type('MockIndexer', (stream.Score,), {})
@@ -454,13 +516,13 @@ class TestLilyPondIndexer(unittest.TestCase):
         expected = mock.MagicMock(spec_set=unicode)
         run_ly = mock.MagicMock()
         # run test
-        with mock.patch('vis.analyzers.indexers.lilypond.outputlilypond') as mock_oly:
+        with mock.patch('vis.analyzers.experimenters.lilypond.outputlilypond') as mock_oly:
             mock_oly.process_score.return_value = expected
             mock_oly.run_lilypond = run_ly
-            with mock.patch('vis.analyzers.indexers.lilypond.oly_settings') as mock_oly_s:
+            with mock.patch('vis.analyzers.experimenters.lilypond.oly_settings') as mock_oly_s:
                 mock_oly_s.LilyPondSettings.return_value = oly_setts
                 with mock.patch('__builtin__.open', mock_open):
-                    actual = lilypond.LilyPondIndexer([mock_score], setts).run()
+                    actual = LilyPondExperimenter([mock_score], setts).run()
         # verify results
             mock_oly.process_score.assert_called_once_with(mock_score, oly_setts)
         mock_open.assert_called_once_with(setts[u'output_pathname'], 'w')
@@ -469,7 +531,7 @@ class TestLilyPondIndexer(unittest.TestCase):
         self.assertEqual(expected, actual)
 
     def test_run_3(self):
-        # with many annotation_parts; without output_pathname; not run_lilypond
+        """with many annotation_parts; without output_pathname; not run_lilypond"""
         # prepare mocks
         mock_open = mock.mock_open()
         mock_score_cls = type('MockIndexer', (stream.Score,), {})
@@ -482,13 +544,13 @@ class TestLilyPondIndexer(unittest.TestCase):
         expected = mock.MagicMock(spec_set=unicode)
         run_ly = mock.MagicMock()
         # run test
-        with mock.patch('vis.analyzers.indexers.lilypond.outputlilypond') as mock_oly:
+        with mock.patch('vis.analyzers.experimenters.lilypond.outputlilypond') as mock_oly:
             mock_oly.process_score.return_value = expected
             mock_oly.run_lilypond = run_ly
-            with mock.patch('vis.analyzers.indexers.lilypond.oly_settings') as mock_oly_s:
+            with mock.patch('vis.analyzers.experimenters.lilypond.oly_settings') as mock_oly_s:
                 mock_oly_s.LilyPondSettings.return_value = oly_setts
                 with mock.patch('__builtin__.open', mock_open):
-                    actual = lilypond.LilyPondIndexer([mock_score], setts).run()
+                    actual = LilyPondExperimenter([mock_score], setts).run()
         # verify results
             mock_oly.process_score.assert_called_once_with(mock_score, oly_setts)
         self.assertEqual(0, mock_open.call_count)
@@ -503,6 +565,6 @@ class TestLilyPondIndexer(unittest.TestCase):
 # Definitions                                                                                      #
 #--------------------------------------------------------------------------------------------------#
 ANNOTATION_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestAnnotationIndexer)
-ANNOTATE_NOTE_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestAnnotateTheNoteIndexer)
-PART_NOTES_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestPartNotesIndexer)
-LILYPOND_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestLilyPondIndexer)
+ANNOTATE_NOTE_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestAnnotateTheNoteExperimenter)
+PART_NOTES_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestPartNotesExperimenter)
+LILYPOND_SUITE = unittest.TestLoader().loadTestsFromTestCase(TestLilyPondExperimenter)
